package com.sece.android;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.HttpConnectionParams;

import android.os.Handler;
import android.os.Message;

/*
 Start a HTTP connection, use http post method to send
 sensor information to SECE server, after create the 
 connection, push the thread into the connection queue
 to make it work asynchronously.  
 */
public class HttpConnection implements Runnable{
	/*To receive status updates from an HttpConnection a Handler is used . 
	The HttpConnection will dispatch a message to the Handler at the occurrence of the following:
HttpConnection.DID_START :
 The connection is removed from the ConnectionManagers queue and started
HttpConnection.DID_SUCCEED: The connection was successful 
HttpConnection.DID_ERROR : The connection failed 
	*/
	public static final int DID_START = 0;
    public static final int DID_ERROR = 1;
    public static final int DID_SUCCEED = 2;
 
    private static final int PUT = 2;
    private static final int POST = 1;
    
    private String url;
    private int method;
    private Handler handler;
    private String data;
 
    private HttpClient httpClient;
    
    public HttpConnection(){
    	this(new Handler());
    }
    
    public HttpConnection(Handler _handler){
    	handler = _handler;
    }
    
    public void create(int method, String url, String data){
    	this.method = method;
    	this.url = url;
    	this.data = data;
    	ConnectionManager.getInstance().push(this);
    }
    
    public void put(String url, String data){
    	create(PUT,url,data);
    }
    
    public void post(String url, String data){
    	create(POST,url,data);
    }
    
    public void run(){
    	handler.sendMessage(Message.obtain(handler,HttpConnection.DID_START));
    	httpClient = new DefaultHttpClient();
    	HttpConnectionParams.setSoTimeout(httpClient.getParams(),25000);
    	try{
    		HttpResponse response = null;    		
    		HttpPost httpPost = new HttpPost(url);
    		//add the authorization header to the Http header
    		httpPost.setHeader("Authorization", "Bearer "+GlobalRecord.code);
            httpPost.setEntity(new StringEntity(data));
            response = httpClient.execute(httpPost);
    		processEntity(response.getEntity());
    	}catch(Exception e){
    		handler.sendMessage(Message.obtain(handler,HttpConnection.DID_ERROR,e));
    	}
    	ConnectionManager.getInstance().didComplete(this);
    }
    
    private void processEntity(HttpEntity entity) throws IllegalStateException,IOException
    {
    	BufferedReader br = new BufferedReader(new InputStreamReader(entity.getContent()));
    	String line, result = "";
    	while((line = br.readLine())!= null)
    		result += line;
    	Message message = Message.obtain(handler,DID_SUCCEED,result);
    	handler.sendMessage(message);
    }
	

}


