package com.sece.android;

import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Binder;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.telephony.PhoneStateListener;
import android.telephony.TelephonyManager;
import android.util.Log;
import android.widget.Toast;

/*Background service, which will be used to initialize the sip stack and the HTTP connection, 
also will be used to register the listeners for the phone state change and Listener for the
the sensor event, it will also allow notification on the status bar of the  phone
to show the service is on or off. We can launch the control pad through the notification.
Meanwhile we get the outgoing call number here. For the sensor state update, we send through the
HTTP Post method in json format here, both for the ambient light sensor and shaking sensor is implemented
by a simple algorithm to judge whether it is shaking or not. With a small state machine to determine the state
update.
*/
public class PhoneStateService extends Service implements SensorEventListener  {

	  private String ip = getLocalIpAddress();
	  private int port = Integer.parseInt("5060");
	  private String username = null;
	  public static SipLayer sipLayer;	
	  private TelephonyManager tm; 
	  private String phonenumber; 
	  private String XML;
	  private NotificationManager mNM;

	    private SensorManager mSensorManager;
		private Sensor mLight;
		private Sensor mAccelerometer;
		private String macAddr;
		private String IsShaked="still";
		private long lastUpdate = -1;
		private float x, y, z;
		private float last_x, last_y, last_z;
		private static final int SHAKE_THRESHOLD = 800;
	 
	//start the service when boot from the very beginnning, that's why we implement a BroadcastReceiver here.
	  public class ServiceReceiver extends BroadcastReceiver {
			@Override
			  public void onReceive(Context context, Intent intent) {
				 try {
                   if (intent.getAction().equals("android.intent.action.NEW_OUTGOING_CALL")) {
			       GlobalRecord.outgoingNumber = intent.getExtras().getString("android.intent.extra.PHONE_NUMBER");
					Log.d("out", GlobalRecord.outgoingNumber);      
                   }

	            } catch (Exception e) {
	            }	
			    TelephonyManager telephony = (TelephonyManager) 
			    context.getSystemService(Context.TELEPHONY_SERVICE);
			    String number = telephony.getLine1Number();
			    MyPhoneStateListener phoneListener=new MyPhoneStateListener(number, context);
			    telephony.listen(phoneListener,PhoneStateListener.LISTEN_CALL_STATE);
			  }
			

		}
	  
	    ServiceReceiver mReceiver = new ServiceReceiver();
	    // Unique Identification Number for the Notification.
	    // We use it on Notification start, and to cancel it.
	    private int NOTIFICATION = 11;
	  
	    public class LocalBinder extends Binder {
	        PhoneStateService getService() {
	            return PhoneStateService.this;
	        }
	    }

	  @Override
	  public void onCreate() {
		  super.onCreate();
		  
		  //create the sensor listener here
		   WifiManager wifiMan = (WifiManager) this.getSystemService(
	                Context.WIFI_SERVICE);
	       WifiInfo wifiInf = wifiMan.getConnectionInfo();
	       macAddr = wifiInf.getMacAddress();
	       mSensorManager = (SensorManager)getSystemService(SENSOR_SERVICE);
	       mAccelerometer = mSensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
	       mLight=mSensorManager.getDefaultSensor(Sensor.TYPE_LIGHT);
		  
		  
		  //create the phone state receiver here
		  IntentFilter intentFilter = new IntentFilter("android.intent.action.PHONE_STATE");
		  registerReceiver(mReceiver, intentFilter);
		  registerReceiver(mReceiver,new IntentFilter("android.intent.action.NEW_OUTGOING_CALL"));
		  mNM = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);

	     // Display a notification about us starting.  We put an icon in the status bar.
	      showNotification();
		  tm = (TelephonyManager) getSystemService(TELEPHONY_SERVICE);
	        phonenumber=tm.getLine1Number();
	    	if (tm.getCallState()==TelephonyManager.CALL_STATE_IDLE)
		    {
	   	    	Log.d("phone","idle"+phonenumber);
		    }
	   	    try{
	   		     sipLayer = new SipLayer(username, ip, port);
	   		}
	   		catch(Throwable e){
	   			 System.out.println("Problem initializing the SIP stack.");
	              e.printStackTrace();
	   		}
	   
	  }
	  //start the service
	  @Override
	  public int onStartCommand(Intent intent, int flags, int startId) {
	      super.onStart(intent, startId);
		  Toast.makeText(this, "service starting", Toast.LENGTH_SHORT).show();
	      System.out.print("Service on");
	      mSensorManager.registerListener(this, mAccelerometer,
	        		SensorManager.SENSOR_DELAY_GAME);    	
	    	mSensorManager.registerListener(this, mLight, SensorManager.SENSOR_DELAY_NORMAL);
	      
	      if(GlobalRecord.PreviousState==TelephonyManager.CALL_STATE_IDLE)
			{
				 XML = 
	        		 GlobalRecord.XML1 +
	        		 "version=\"1\"\n" +
	        		 " state=\"full\"\n" +
	        		 "entity=\"tel:+"  +phonenumber + "\">\n" +
	        		 "</dialog-info>";
	        	 System.out.print(XML);
	            try{
	        	     sipLayer.sendMessage(XML);	
	        	 }
	        	 catch(Throwable e){
	        		 Log.e("sip","ERROR sending message: " + e.getMessage() + "\n");
	        	 }
			}
	      
	      // If we get killed, after returning from here, restart
	      return START_STICKY;
	  }
	  //stop the service
	  @Override
	  public void onDestroy() {
		  super.onDestroy();
		  mSensorManager.unregisterListener(this);
		  mNM.cancel(NOTIFICATION);
		  unregisterReceiver(mReceiver);
		  GlobalRecord.Dialog_id=1;
		  GlobalRecord.Version=1;
		  sipLayer.sipStack.stop();
	    Toast.makeText(this, "service done", Toast.LENGTH_SHORT).show();
	    System.out.print("Service off");
	  }
	  //we use this method to get the LocalIpAddress
	  public String getLocalIpAddress() {   
	        try {   
	            for (Enumeration<NetworkInterface> en = NetworkInterface   
	                    .getNetworkInterfaces(); en.hasMoreElements();) {   
	                NetworkInterface intf = en.nextElement();   
	                for (Enumeration<InetAddress> enumIpAddr = intf   
	                        .getInetAddresses(); enumIpAddr.hasMoreElements();) {   
	                    InetAddress inetAddress = enumIpAddr.nextElement();   
	                    if (!inetAddress.isLoopbackAddress()) {   
	                        return inetAddress.getHostAddress().toString();   
	                    }   
	                }   
	            }   
	        } catch (SocketException ex) {   
	            Log.e("WifiPreference IpAddress", ex.toString());   
	        }   
	        return null;   
	    }
	  
	  @Override
	    public IBinder onBind(Intent intent) {
	        return mBinder;
	    }

	    // This is the object that receives interactions from clients.  
	    private final IBinder mBinder = new LocalBinder();
	
	    
	    /**
	     * Show a notification while this service is running.
	     */
	    private void showNotification() {
	    	// In this sample, we'll use the same text for the ticker and the expanded notification
	        CharSequence text = "PhoneStateService Start";

	        // Set the icon, scrolling text and timestamp
	        Notification notification = new Notification(R.drawable.ic_launcher, text,
	                System.currentTimeMillis());
	        Intent notificationIntent = new Intent(this, SECEserviceActivity.class);
	       
	        // The PendingIntent to launch our activity if the user selects this notification
	        PendingIntent contentIntent = PendingIntent.getActivity(this, 0, notificationIntent, 0);

	        // Set the info for the views that show in the notification panel.
	        notification.setLatestEventInfo(this, "PhoneStateService",
	                       text, contentIntent);

	        // Send the notification.
	        mNM.notify(NOTIFICATION, notification);
	    }
	    public void onAccuracyChanged(Sensor sensor, int accuracy){
	    	if(sensor.getType() == Sensor.TYPE_LIGHT){    		
	    		    Log.i("Sensor Changed", "Accuracy :" + accuracy);    		
	       }
	    }
	    
	    public void onSensorChanged(SensorEvent event) {
	    	
	    	 Handler handler = new Handler(){
	 	    	public void handleMessage(Message message){
	 	    		switch (message.what) {
	 	            case HttpConnection.DID_START: {
	 	              Log.i("HTTP","Starting connection...");
	 	              break;
	 	            }
	 	            case HttpConnection.DID_SUCCEED: {
	 	              String response = (String) message.obj;
	 	              Log.i("HTTP",response);
	 	              break;
	 	            }
	 	            case HttpConnection.DID_ERROR: {
	 	              Exception e = (Exception) message.obj;
	 	              e.printStackTrace();
	 	              Log.i("HTTP","Connection failed.");
	 	              break;
	 	            }
	 	          }
	 	    	}
	 	    };
	    	
	    	
	    	
	    	  if( event.sensor.getType() == Sensor.TYPE_LIGHT){    	
	    	  
	    	   float percent = (float) (event.values[0]/10240.0);
	    	   if(percent<0.001)
	    		   percent = 0;
	    	   percent*=100;	    	   
	    	   String json="{\"m\": [ { \"n\": "+"\"ambient_light\" , \"u\": %, \"v\": "+percent+
	    	    " }]}"; 	    	
	    	  }
	    	  else if(event.sensor.getType()==Sensor.TYPE_ACCELEROMETER)
	    	  {
	    		  
	    		  long curTime = System.currentTimeMillis();
	    		    // only allow one update every 100ms.
	    		    if ((curTime - lastUpdate) >= 100) {
	    			long diffTime = (curTime - lastUpdate);
	    			lastUpdate = curTime;
	    	 //calculate to determine the shaking motion
	    			x = event.values[SensorManager.DATA_X];
	    			y = event.values[SensorManager.DATA_Y];
	    			z = event.values[SensorManager.DATA_Z];
	    	 
	    			float speed = Math.abs(x+y+z - last_x - last_y - last_z)
	    	                              / diffTime * 10000;
	    			if (speed >= SHAKE_THRESHOLD) {
	    			    // yes, this is a shake action! Do something about it!
	    				IsShaked = "shaking";
	    			}
	    			else {
	    				IsShaked = "still";
	    			}
	    			if(IsShaked!=GlobalRecord.PreviousState1)
	    			{
	    				if(IsShaked == "shaking")
	    				new HttpConnection(handler).post("http://lagrange.cs.columbia.edu/sensor?username=sece&password=senseeverything&me.phone.state=shaking", "");	    				
	    				Log.i("sensor",IsShaked);
	    				GlobalRecord.PreviousState1 = IsShaked;
	    			}
	    			
	    				
	    			last_x = x;
	    			last_y = y;
	    			last_z = z;
	    			
	    	  }
	    	 }
	    }
	  
	  
	}
