package com.sece.android;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Properties;
import java.util.TooManyListenersException;

import javax.sip.DialogTerminatedEvent;
import javax.sip.IOExceptionEvent;
import javax.sip.InvalidArgumentException;
import javax.sip.ListeningPoint;
import javax.sip.ObjectInUseException;
import javax.sip.PeerUnavailableException;
import javax.sip.RequestEvent;
import javax.sip.ResponseEvent;
import javax.sip.ServerTransaction;
import javax.sip.SipException;
import javax.sip.SipFactory;
import javax.sip.SipListener;
import javax.sip.SipProvider;
import javax.sip.SipStack;
import javax.sip.TimeoutEvent;
import javax.sip.TransactionTerminatedEvent;
import javax.sip.TransportNotSupportedException;
import javax.sip.address.Address;
import javax.sip.address.AddressFactory;
import javax.sip.address.SipURI;
import javax.sip.header.AuthorizationHeader;
import javax.sip.header.CSeqHeader;
import javax.sip.header.CallIdHeader;
import javax.sip.header.ContactHeader;
import javax.sip.header.ContentTypeHeader;
import javax.sip.header.FromHeader;
import javax.sip.header.HeaderFactory;
import javax.sip.header.MaxForwardsHeader;
import javax.sip.header.ToHeader;
import javax.sip.header.ViaHeader;
import javax.sip.message.MessageFactory;
import javax.sip.message.Request;
import javax.sip.message.Response;

import android.util.Log;

/*
 * The implementation of the SIP protocol, we initialize the SIP protocol here
 * define the function of sending PUBLISH Request, process response to receive
 * the response successfully.
 * Initialization of the SipFactory, and the corresponding sip header. Can be
 * used conveniently to send message through sip protocol.
 * */
public class SipLayer implements SipListener {

    private String username;

    public SipStack sipStack;

    private SipFactory sipFactory;

    private AddressFactory addressFactory;

    private HeaderFactory headerFactory;

    private MessageFactory messageFactory;

    private SipProvider sipProvider;
    
    
 
    public SipLayer(String username, String ip, int port)
    throws PeerUnavailableException, TransportNotSupportedException,
    InvalidArgumentException, ObjectInUseException,
    TooManyListenersException {
    setUsername(username);
   
    sipFactory = SipFactory.getInstance();
	Properties properties = new Properties();
	properties.setProperty("javax.sip.STACK_NAME", "SIP_Activity");
	properties.setProperty("javax.sip.IP_ADDRESS", ip);

	sipStack = sipFactory.createSipStack(properties);
	headerFactory = sipFactory.createHeaderFactory();
	addressFactory = sipFactory.createAddressFactory();
	messageFactory = sipFactory.createMessageFactory();

	ListeningPoint tcp = sipStack.createListeningPoint(port, "tcp");

      sipProvider = sipStack.createSipProvider(tcp);
      sipProvider.addSipListener(this);

}
    
    public void sendMessage(String message) throws ParseException,
    InvalidArgumentException, SipException {

    		SipURI from = addressFactory.createSipURI(getUsername(), getHost()
    					+ ":" + getPort());
    		Address fromNameAddress = addressFactory.createAddress(from);
    		fromNameAddress.setDisplayName(getUsername());
    		FromHeader fromHeader = headerFactory.createFromHeader(fromNameAddress,
    			"Sip_Android");
    		//sip name and sece server ip address
    		String username = "android";
    		String address = "128.59.21.160:5060";

    		SipURI toAddress = addressFactory.createSipURI(username, address);
    		Address toNameAddress = addressFactory.createAddress(toAddress);
    		toNameAddress.setDisplayName(username);
    		ToHeader toHeader = headerFactory.createToHeader(toNameAddress, null);

    		SipURI requestURI = addressFactory.createSipURI(username, address);
    		requestURI.setTransportParam("tcp");

    		ArrayList<ViaHeader> viaHeaders = new ArrayList<ViaHeader>();
    		ViaHeader viaHeader = headerFactory.createViaHeader(getHost(),
    				getPort(), "tcp", "branch1");
    		viaHeaders.add(viaHeader);
    		//The authorization header which will include the registration code
    		//we get form the registration website
    		AuthorizationHeader auHeader=headerFactory.createAuthorizationHeader("Bearer "+GlobalRecord.code);

    		CallIdHeader callIdHeader = sipProvider.getNewCallId();
    		//indicate we are sending PUBLISH Request
    		CSeqHeader cSeqHeader = headerFactory.createCSeqHeader(GlobalRecord.CSeq++,
    				Request.PUBLISH);

    		MaxForwardsHeader maxForwards = headerFactory
    				.createMaxForwardsHeader(70);

    		Request request = messageFactory.createRequest(requestURI,
			Request.PUBLISH, callIdHeader, cSeqHeader, fromHeader,
			toHeader, viaHeaders, maxForwards);
    		request.addHeader(auHeader);

		SipURI contactURI = addressFactory.createSipURI(getUsername(),
			getHost());
		contactURI.setPort(getPort());
		Address contactAddress = addressFactory.createAddress(contactURI);
		contactAddress.setDisplayName(getUsername());
		ContactHeader contactHeader = headerFactory
			.createContactHeader(contactAddress);
			request.addHeader(contactHeader);

			ContentTypeHeader contentTypeHeader = headerFactory
				.createContentTypeHeader("application", "pidf+xml");
			request.setContent(message, contentTypeHeader);

			sipProvider.sendRequest(request);
    		}
    
    public void processResponse(ResponseEvent evt) {
    	Response response = evt.getResponse();
    	int status = response.getStatusCode();

    	if ((status >= 200) && (status < 300)) { //Success!
    	    System.out.println("succeed");
    		Log.d("sip", "--Sent");
    	    return;
    	}

    	Log.e("sip", "Previous message not sent: " + status);
        }
    
    public void processRequest(RequestEvent evt) {
    	Request req = evt.getRequest();

    	String method = req.getMethod();
    	if (!method.equals("PUBLISH")) { //bad request type.
    	    Log.e("Sip","Bad request type: " + method);
    	    return;
    	}

    	FromHeader from = (FromHeader) req.getHeader("From");
    	Log.d("Sip",new String(req.getRawContent()));
    	Response response = null;
    	try { //Reply with OK
    	    response = messageFactory.createResponse(200, req);
    	    ToHeader toHeader = (ToHeader) response.getHeader(ToHeader.NAME);
    	    toHeader.setTag("888"); //This is mandatory as per the spec.
    	    ServerTransaction st = sipProvider.getNewServerTransaction(req);
    	    st.sendResponse(response);
    	} catch (Throwable e) {
    	    e.printStackTrace();
    	    Log.e("sip","Can't send OK reply.",e);
    	}
        }
    
    public void processTimeout(TimeoutEvent evt) {
    	Log.e("Sip","Previous message not sent: " + "timeout");
        }

        /** 
         * This method is called by the SIP stack when there's an asynchronous
         * message transmission error.  
         */
        public void processIOException(IOExceptionEvent evt) {
    	Log.e("SIP","Previous message not sent: "
    		+ "I/O Exception");
        }

        /** 
         * This method is called by the SIP stack when a dialog (session) ends. 
         */
        public void processDialogTerminated(DialogTerminatedEvent evt) {
        }

        /** 
         * This method is called by the SIP stack when a transaction ends. 
         */
        public void processTransactionTerminated(TransactionTerminatedEvent evt) {
        }
    
    
    	public String getHost() {
    	int port = sipProvider.getListeningPoint().getPort();
    	String host = sipStack.getIPAddress();
    	return host;
        }

        public int getPort() {
    	int port = sipProvider.getListeningPoint().getPort();
    	return port;
        }

        public String getUsername() {
    	return username;
        }

        public void setUsername(String newUsername) {
    	username = newUsername;
        }
}
